#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#define VENDOR_RQ_SET_LED    0
#define set  0
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_GET_LIGHT  2
#define VENDOR_RQ_GET_LED2   3
#define VENDOR_RQ_GET_LED3   5
#define VENDOR_RQ_GET_AUTO   4
char in_c(char pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return ((PINC & (1<<pin))>>pin);
}

void out_c(char pin, char val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if(val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}
void out_b(char pin, char val)
{
    // write logic level to port-C's pin
    DDRB |= 1<<pin;
    if(val)
        PORTB |= 1<<pin;
    else
        PORTB &= ~(1<<pin);
}
uint16_t read_adc(uint8_t channel)
{
    // read analog value from specified channel of port C
    ADMUX = 0x40;
    ADMUX |= channel;
    ADCSRA = 0xC6;
    while ((ADCSRA & (1<<ADSC)))
        ;
    return ADCL + ADCH*256;
}

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[2];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        out_c(0,1);
        out_c(1,1);
        out_c(2,1);
        out_b(0,1);
        out_b(1,1);
        out_b(2,1);
        out_b(3,1);
        out_b(4,1);
        
        return 0;
        // read voltage from light sensor
    }
    else if(rq->bRequest == VENDOR_RQ_GET_LED3)
    {
	uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_b(led_no, led_val);
        return 0;
    }
    else if(rq->bRequest == VENDOR_RQ_GET_LED2)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_c(led_no, led_val);
	return 0;
    }
    else if(rq->bRequest == VENDOR_RQ_GET_AUTO)
    {
        uint16_t x = read_adc(4);
        if (x > 700)
        {
            out_c(0,0);
            out_c(1,0);
            out_c(2,0);
            out_b(0,0);
            out_b(1,0);
            out_b(2,0);
            out_b(3,0);
            out_b(4,0);
            
        }
        else if (x > 500)
        {
            out_c(0,0);
            out_c(1,1);
            out_c(2,0);
            out_b(0,1);
            out_b(1,0);
            out_b(2,1);
            out_b(3,0);
            out_b(4,1);
            
        }
        else //if (x > 300)
        {
            out_c(0,1);
            out_c(1,1);
            out_c(2,1);
            out_b(0,1);
            out_b(1,1);
            out_b(2,1);
            out_b(3,1);
            out_b(4,1);
        }
       return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_c(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT)
    {
        dataBuffer[0] = read_adc(4)/256;
        dataBuffer[1] = read_adc(4)%256;
	usbMsgPtr = dataBuffer;
	return 2;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}
int main()
{
    DDRC  = 0b00000111;  // PC0..PC2 = Output, PC3..PC5 = Input
    PORTC = 0b00000000;  // PC7..PC0 = 0
    PORTC |= (1 << 3);   // Enable pull-up at PC3

    DDRD  = 0b00001000;
    PORTD = 0b00000000;
    usbInit();
    usbDeviceDisconnect();
    _delay_ms(300);     /* fake USB disconnect for > 250 ms */
    usbDeviceConnect();
    sei();              /* enable interrupts */
    while (1)           /* main event loop */
    {
        usbPoll();      /* คำสั่งนี้ต้องถูกเรียกอย่างน้อยที่สุดทุก ๆ 50ms */
    }
    return 0;
}
